import React, {
  useState,
  useCallback,
  useEffect,
} from 'react';
import ReactDOM from 'react-dom';
import {
  arc,
  pie,
  scaleBand,
  scaleLinear,
  max,
  format,
} from 'd3';

import { useData } from './useData';
import { AxisBottom } from './AxisBottom';
import { AxisLeft } from './AxisLeft';
import { Marks } from './Marks';

const width = 960;
const height = 500;
const margin = {
  top: 30,
  right: 20,
  bottom: 70,
  left: 200,
};
const xAxisLableOffset = 50;
const xAxisLableOffsetX = 60;
const siFormat = format('.2s');
const xAxisTickFormat = (tickValue) =>
  siFormat(tickValue).replace('G', 'B');

const App = () => {
  const data = useData();

  if (!data) {
    return <pre>Loading...</pre>;
  }

  const innerHeight = height - margin.top - margin.bottom;
  const innerWidth = width - margin.left - margin.right;

  const yValue = (d) => d.Country;
  const xValue = (d) => d.Population;

  const yScale = scaleBand()
    .domain(data.map(yValue))
    .range([0, innerHeight])
    .paddingInner(0.1);

  const xScale = scaleLinear()
    .domain([0, max(data, xValue)])
    .range([0, innerWidth]);

  return (
    <svg width={width} height={height}>
      <g
        transform={`translate(${margin.left},${margin.top})`}
      >
        <AxisBottom
          xScale={xScale}
          innerHeight={innerHeight}
          tickFormat={xAxisTickFormat}
        />

        <AxisLeft yScale={yScale} />
        <text
          className="label"
          x={innerWidth / 2 - xAxisLableOffsetX}
          y={innerHeight + xAxisLableOffset}
          textAnchor="middle"
        >
          Population
        </text>
        <Marks
          data={data}
          yScale={yScale}
          xScale={xScale}
          yValue={yValue}
          xValue={xValue}
          tooltipFormat={xAxisTickFormat}
        />
      </g>
    </svg>
  );
};

const rootElement = document.getElementById('root');
ReactDOM.render(<App />, rootElement);
